uniform vec4 diffuseColor;
uniform int hasTexture;
uniform float incandescence;
uniform vec3 lightDirection;
uniform int receiveShadows;
uniform vec2 textureScale;
uniform vec2 textureOffset;

#if defined(PT_PCF_SHADOWS)
uniform sampler2DShadow shadowMapTexture;
#else
uniform sampler2D shadowMapTexture;
#endif

varying vec2 texCoord;
varying vec4 lightViewportTexCoordDivW;
varying vec3 worldPosition;
varying vec3 worldNormal;

void main() {
    float shadow = 1.0;
    
	if(receiveShadows == 1) {
    	float bias = max(0.0025 * (1.0 - dot(worldNormal, lightDirection)), 0.0005);

#ifdef PT_PCF_SHADOWS
#ifdef GL_EXT_shadow_samplers 
        shadow = shadow2DProjEXT(shadowMapTexture, lightViewportTexCoordDivW + vec4(0, 0, -bias / lightViewportTexCoordDivW.w, 0));
#else
        shadow = shadow2DProj(shadowMapTexture, lightViewportTexCoordDivW + vec4(0, 0, -bias / lightViewportTexCoordDivW.w, 0)).z;
#endif
#else
        float depth = texture2DProj(shadowMapTexture, lightViewportTexCoordDivW + vec4(0, 0, -bias / lightViewportTexCoordDivW.w, 0)).x;
        float R = (lightViewportTexCoordDivW + vec4(0, 0, -bias / lightViewportTexCoordDivW.w, 0)).p / (lightViewportTexCoordDivW + vec4(0, 0, -bias / lightViewportTexCoordDivW.w, 0)).q;
            
        shadow = (R <= depth) ? 1.0 : 0.0;
#endif
	}

	float inc = mix(0.5, 0.0, incandescence);
    
	if (hasTexture == 1) {
		gl_FragColor = texture2D(CC_Texture0, texCoord / textureScale + textureOffset) * diffuseColor * mix(vec4(1.0, 1.0, 1.0, 0.0), vec4(shadow, shadow, shadow, 1.0), vec4(inc, inc, inc, 1.0));  // was 0.5
	}
	else {
		gl_FragColor = diffuseColor * mix(vec4(1.0, 1.0, 1.0, 0.0), vec4(shadow, shadow, shadow, 1.0), vec4(inc, inc, inc, 1.0));  // was 0.5
	}
}
